%---------------------------------------------------------------------------------------------
%--- Original Euler equation: c0(st)*gpit + c1(st)*gpitm1 - Etatp1 *gpit + gsigma(st)*gepst 
%---------------------------------------------------------------------------------------------
close all
clear all
clc

%--- Parameter values
P = [ 
  0.95 0.15 
  0.05 0.85];
c0 = [1.25; 0.96];
%c0 = [2.5; 2.5];
c1 = [0.0; 0.0];
gsigma = [0.1; 0.6];
%gsigma = [0.3; 0.3];
%--- Initial values
gpitm1 = 0.01;  %1% quarterly inflation.
gepst = 1.0;    %1 std deviation.


%--------------------------------------------------------------------
%--- First-order approximation: 
%--- Expected Euler equation error: E_{t} Euler equation = 0.
%--- Given: gpit = a(st) * gpitm1 + gxi(st) * gepst, and Etatp1 = P[1,st]*a(1) + P[2,st]*a(2)
%---        c0(st)*gpit + c1(st)*gpitm1 - Etatp1 *gpit + gsigma(st)*gepst 
%---    ==> ( a(st)*(c0(st)-Etatp1)+c1(st) ) * gpitm1 + ( gxi(st)*(c0(st)-Etatp1)+gsigma(st) ) * gepst

%--------------------------------------------
%---- With our partition (correct) method  
%---     for estimates: gpit = a(st) * gpitm1 + gxi(st) * gepst
%--------------------------------------------
a = [0.0; 0.0];
gxi = [-0.08; -0.625];
%---
errors_mean = 0;
disp('************* Euler equation errors for correct partition at regime 1: ***************');
st = 1;
Etatp1 = P(1,st)*a(1) + P(2,st)*a(2);
coeff4pi = ( a(st)*(c0(st)-Etatp1)+c1(st) )*gpitm1
coeff4eps = ( gxi(st)*(c0(st)-Etatp1)+gsigma(st) )*gepst
errors_mean = errors_mean + abs(coeff4pi) + abs(coeff4eps);
disp('************* Euler equation errors for correct partition at regime 2: ***************');
st = 2;
Etatp1 = P(1,st)*a(1) + P(2,st)*a(2);
coeff4pi = ( a(st)*(c0(st)-Etatp1)+c1(st) )*gpitm1
coeff4eps = ( gxi(st)*(c0(st)-Etatp1)+gsigma(st) )*gepst
errors_mean = errors_mean + abs(coeff4pi) + abs(coeff4eps);
disp('************* Overall Euler equation errors for correct partition (1st-order approximation = exact solution): ***************');
errors_mean = errors_mean/2
disp('--- The log10 value for the above error: ---');
log10(errors_mean)


%--------------------------------------------
%---- With naive method---the 1st-order estimation:
%---     gpit = a(st) * gpitm1 + gxi(st) * gepst
%--------------------------------------------
a = [0.0;  0.0];
gxi = [-0.191083; -0.191083];
%---
errors_mean = 0;
disp(' ');
disp(' ');
disp('************* Euler equation errors for naive method at regime 1: ***************');
st = 1;
Etatp1 = P(1,st)*a(1) + P(2,st)*a(2);
coeff4pi = ( a(st)*(c0(st)-Etatp1)+c1(st) )*gpitm1
coeff4eps = ( gxi(st)*(c0(st)-Etatp1)+gsigma(st) )*gepst
errors_mean = errors_mean + abs(coeff4pi) + abs(coeff4eps);
disp('************* Euler equation errors for naive method at regime 2: ***************');
st = 2;
Etatp1 = P(1,st)*a(1) + P(2,st)*a(2);
coeff4pi = ( a(st)*(c0(st)-Etatp1)+c1(st) )*gpitm1
coeff4eps = ( gxi(st)*(c0(st)-Etatp1)+gsigma(st) )*gepst
errors_mean = errors_mean + abs(coeff4pi) + abs(coeff4eps);
disp('************* Overall Euler equation errors for naive method (1st-order approximation): ***************');
errors_mean = errors_mean/2.0
disp('--- The log10 value for the above error: ---');
log10(errors_mean)

%--------------------------------------------
%---- With naive method---the 2nd-order estimation:
%---     gpit = a(st) * gpitm1 + gxi(st) * gepst + ...
%---            0.5*( b1(st)*gpitm1^2 + b2(st)*gpitm1*gepst + b3(st)*gpitm1*gchi + ...
%---                  + b4(st)*gepst^2 + b5(st)*gpepst*gchi + b6(st)*gchi^2
%--------------------------------------------
gchi = 1.0;
Etepstp1sq = 1.0;
b1 = [0.0; 0.0];
b2 = [0.0+0.0; 0.0+0.0];
b3 = [2*0.0;  2*0.0];
b4 = [0.0; 0.0];
b5 = [0.117922*2;  -0.353767*2];
b6 = [0.0; 0.0];

errors_mean = 0;
disp(' ');
disp(' ');
disp('************* Euler equation errors for naive method (2nd-order approximation) at regime 1: ***************');
st = 1;
Etatp1 = P(1,st)*a(1) + P(2,st)*a(2);
Etb1tp1 = P(1,st)*b1(1) + P(2,st)*b1(2);
Etb2tp1 = P(1,st)*b2(1) + P(2,st)*b2(2);
Etb3tp1 = P(1,st)*b3(1) + P(2,st)*b3(2);
Etb4tp1 = P(1,st)*b4(1) + P(2,st)*b4(2);
Etb5tp1 = P(1,st)*b5(1) + P(2,st)*b5(2);
Etb6tp1 = P(1,st)*b6(1) + P(2,st)*b6(2);
gpit = a(st)*gpitm1 + gxi(st)*gepst + ...
       0.5*( b1(st)*gpitm1^2 + b2(st)*gpitm1*gepst + b3(st)*gpitm1*gchi + ...
             + b4(st)*gepst^2 + b5(st)*gepst*gchi + b6(st)*gchi^2 );
Etgpitp1 = Etatp1*gpit + Etb1tp1*gpit^2 + Etb3tp1*gpit*gchi + Etb4tp1*1.0 + Etb4tp1*Etepstp1sq + Etb4tp1*gchi^2;
errors_2nd_regime1 = c0(st)*gpit + c1(st)*gpitm1 - Etatp1 *gpit + gsigma(st)*gepst 
errors_mean = errors_mean + abs(errors_2nd_regime1);
disp('************* Euler equation errors for naive method (2nd-order approximation) at regime 2: ***************');
st = 2;
Etatp1 = P(1,st)*a(1) + P(2,st)*a(2);
Etb1tp1 = P(1,st)*b1(1) + P(2,st)*b1(2);
Etb2tp1 = P(1,st)*b2(1) + P(2,st)*b2(2);
Etb3tp1 = P(1,st)*b3(1) + P(2,st)*b3(2);
Etb4tp1 = P(1,st)*b4(1) + P(2,st)*b4(2);
Etb5tp1 = P(1,st)*b5(1) + P(2,st)*b5(2);
Etb6tp1 = P(1,st)*b6(1) + P(2,st)*b6(2);
gpit = a(st)*gpitm1 + gxi(st)*gepst + ...
       0.5*( b1(st)*gpitm1^2 + b2(st)*gpitm1*gepst + b3(st)*gpitm1*gchi + ...
             + b4(st)*gepst^2 + b5(st)*gepst*gchi + b6(st)*gchi^2 );
Etgpitp1 = Etatp1*gpit + Etb1tp1*gpit^2 + Etb3tp1*gpit*gchi + Etb4tp1*1.0 + Etb4tp1*Etepstp1sq + Etb4tp1*gchi^2;
errors_2nd_regime2 = c0(st)*gpit + c1(st)*gpitm1 - Etatp1 *gpit + gsigma(st)*gepst 
errors_mean = errors_mean + abs(errors_2nd_regime2);
disp('************* Overall Euler equation errors for naive method (2nd-order approximation): ***************');
errors_mean = errors_mean/2.0
disp('--- The log10 value for the above error: ---');
log10(errors_mean)